/*
    libparted - a library for manipulating disk partitions
    Copyright (C) 1998-2000 Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <parted/parted.h>
#include <sys/types.h>
#include <pwd.h>
#include <unistd.h>
#include <grp.h>
#include <stdlib.h>

#include <libintl.h>
#if ENABLE_NLS
#  define _(String) gettext (String)
#else
#  define _(String) (String)
#endif /* ENABLE_NLS */

/* ped_malloc() debugging.  Stick the address and size of memory blocks that
 * weren't ped_free()d in here, and an exception will be thrown when it is
 * allocated.  That way, you can find out what, exactly, the allocated thing
 * is, and where it is created.
 */
static int	dodgey_malloc_list[][2] = {
 {0,		0}
};

/* Note: loop must be added AFTER msdos, because DOS partition tables and 
 * FAT file systems are hard to tell apart :-)
 */
static void
init_disk_types ()
{
	ped_disk_msdos_init ();
//	ped_disk_loop_init ();
}

static void
init_file_system_types ()
{
	ped_file_system_linux_swap_init ();
	ped_file_system_ext2_init ();
	ped_file_system_fat_init ();
}

int
ped_init ()
{
	if (getlogin())
		initgroups (getlogin(), 0);

	init_disk_types ();
	init_file_system_types ();

	return 1;
}

static void
done_disk_types ()
{
	ped_disk_msdos_done ();
//	ped_disk_loop_done ();
}

static void
done_file_system_types ()
{
	ped_file_system_fat_done ();
	ped_file_system_ext2_done ();
	ped_file_system_linux_swap_done ();
}

void
ped_done ()
{
	ped_device_free_all ();

	done_disk_types ();
	done_file_system_types ();
}

const char*
ped_get_version ()
{
	return VERSION;
}

void*
ped_malloc (size_t size)
{
	void*		mem;
	int		i;

	mem = (void*) malloc (size);
	if (!mem) {
		ped_exception_throw (PED_EXCEPTION_FATAL, PED_EXCEPTION_CANCEL,
				     _("Out of memory."));
		return NULL;
	}
	memset (mem, 0xff, size);	/* assist debugging */

	for (i=0; dodgey_malloc_list[i][0]; i++) {
		if (dodgey_malloc_list[i][0] == (int) mem
		    && dodgey_malloc_list[i][1] == size) {
			ped_exception_throw (
				PED_EXCEPTION_INFORMATION,
				PED_EXCEPTION_OK,
				"Dodgey malloc(%x) == %p occured",
				size, mem);
			break;
		}
	}

	return mem;
}

int
ped_realloc (void** old, size_t size)
{
	void*		mem;

	mem = (void*) realloc (*old, size);
	if (!mem) {
		ped_exception_throw (PED_EXCEPTION_FATAL, PED_EXCEPTION_CANCEL,
				     _("Out of memory."));
		return 0;
	}
	*old = mem;
	return 1;
}

void
ped_free (void* ptr)
{
	PED_ASSERT (ptr != NULL, return);
	PED_ASSERT (ptr != ((void*) -1), return);

	free (ptr);
}
